import {EventBus, Events} from '../../../EventBus'
let CellDefault = pillar.vuecomponents.table.cells.renderer.CellDefault;

const TEMPLATE =`
<div>
    <div class="tasks">
        <a
            v-for="t in tasks"
            :class="taskClass(t)"
            :href="taskLink(t)"
            :key="t._id"
            :title="taskTitle(t)"
            @click.prevent="onTaskClicked(t)"
        />
    </div>
    <button class="add-task-link"
        v-if="canAddTask"
        @click.prevent="onAddTask"
    >
    <i class="pi-plus">Task</i>
    </button>
</div>
`;

let CellTasks = Vue.component('attract-cell-tasks', {
    extends: CellDefault,
    template: TEMPLATE,
    computed: {
        tasks() {
            return this.rawCellValue;
        },
        canAddTask() {
            if (this.tasks.length < 1 ) {
                let projectId = ProjectUtils.projectId();
                return attract.auth.AttractAuth.canUserCreateTask(projectId);
            }
            return false;
        }
    },
    data() {
        return {
            selected_task_id: null
        }
    },
    created() {
        EventBus.$on(Events.DESELECT_ITEMS, this.deselectTask);
    },
    beforeDestroy() {
        EventBus.$off(Events.DESELECT_ITEMS, this.deselectTask);
    },
    methods: {
        taskClass(task) {
            let classes = {'active': this.selected_task_id === task._id};
            classes[`task status-${task.properties.status}`] = true;
            return classes;
        },
        taskLink(task) {
            let project_url = ProjectUtils.projectUrl();
            let node_type = this.rowObject.underlyingObject.node_type;
            let item_type = node_type.replace('attract_', '') + 's'; // eg. attract_asset to assets
            return `/attract/${project_url}/${item_type}/with-task/${task._id}`;
        },
        taskTitle(task) {
            let status = (task.properties.status || '').replace('_', ' ');
            return `Task: ${task.name}\nStatus: ${status}`
        },
        onTaskClicked(task) {
            task_open(task._id, ProjectUtils.projectUrl());
            EventBus.$emit(Events.DESELECT_ITEMS);
            this.selected_task_id = task._id;
        },
        onAddTask(event) {
            task_create(this.rowObject.getId(), this.column.taskType);
        },
        deselectTask() {
            this.selected_task_id = null;
        }
    },
});

export { CellTasks }
