let ColumnBase = pillar.vuecomponents.table.columns.ColumnBase;
let CellPrettyDate = pillar.vuecomponents.table.cells.renderer.CellPrettyDate;

function firstDate(prevDate, task) {
    let candidate = task.properties.due_date;
    if (prevDate && candidate) {
        if (prevDate !== candidate) {
            return new Date(candidate) < new Date(prevDate) ? candidate : prevDate;
        }
    }
    return prevDate || candidate;
}

function lastDate(prevDate, task) {
    let candidate = task.properties.due_date;
    if (prevDate && candidate) {
        if (prevDate !== candidate) {
            return new Date(candidate) > new Date(prevDate) ? candidate : prevDate;
        }
    }
    return prevDate || candidate;
}

function nextDate(prevDate, task) {
    let candidate = task.properties.due_date;
    if(candidate && new Date(candidate) >= new Date()) {
        return firstDate(prevDate, task);
    }
    return prevDate;
}

class DueDate extends ColumnBase {
    getCellRenderer(rowObject) {
        return CellPrettyDate.options.name;
    }

    getCellClasses(dueDate, rowObject) {
        let classes = super.getCellClasses(dueDate, rowObject);
        let isPostDueDate = false;
        if (dueDate) {
            isPostDueDate = new Date(dueDate) < new Date();
        }
        classes['warning'] = isPostDueDate;
        return classes;
    }

    compareRows(rowObject1, rowObject2) {
        let dueDateStr1 = this.getRawCellValue(rowObject1);
        let dueDateStr2 = this.getRawCellValue(rowObject2);
        if (dueDateStr1 === dueDateStr2) return 0;
        if (dueDateStr1 && dueDateStr2) {
            return new Date(dueDateStr1) < new Date(dueDateStr2) ? -1 : 1;
        }
        return dueDateStr1 ? -1 : 1;
    }
    
    getCellTitle(rawCellValue, rowObject) {
        return rawCellValue;
    }
}

export class FirstTaskDueDate extends DueDate {
    constructor() {
        super('First Due Date', 'first-duedate');
    }
    getRawCellValue(rowObject) {
        let tasks = rowObject.tasks || [];
        return tasks.reduce(firstDate, undefined) || '';
    }
}

export class LastTaskDueDate extends DueDate {
    constructor() {
        super('Last Due Date', 'last-duedate');
    }
    getRawCellValue(rowObject) {
        let tasks = rowObject.tasks || [];
        return tasks.reduce(lastDate, undefined) || '';
    }
}

export class NextTaskDueDate extends DueDate {
    constructor() {
        super('Next Due Date', 'next-duedate');
    }
    getRawCellValue(rowObject) {
        let tasks = rowObject.tasks || [];
        return tasks.reduce(nextDate, undefined) || '';
    }
}

export class TaskDueDate extends DueDate {
    constructor() {
        super('Due Date', 'duedate');
    }
    getRawCellValue(rowObject) {
        let task = rowObject.getTask();
        return task.properties.due_date || '';
    }
}
