/**
 * Helper class that listens to events triggered when a RowObject task is updated/created/deleted and keep the tasks
 * array of a RowObject up to date accordingly.
 */

export class TaskEventListener {
    constructor(rowWithTasks) {
        this.rowObject = rowWithTasks;
    }

    register() {
        pillar.events.Nodes.onParentCreated(this.rowObject.getId(), 'attract_task', this.onTaskCreated.bind(this));
        this.rowObject.tasks.forEach(this.registerEventListeners.bind(this));
    }

    registerEventListeners(task) {
        pillar.events.Nodes.onUpdated(task._id, this.onTaskUpdated.bind(this));
        pillar.events.Nodes.onDeleted(task._id, this.onTaskDeleted.bind(this));
    }

    onTaskCreated(event, newTask) {
        this.registerEventListeners(newTask);
        this.rowObject.tasks = this.rowObject.tasks.concat(newTask);
    }

    onTaskUpdated(event, updatedTask) {
        this.rowObject.tasks = this.rowObject.tasks.map((t) => {
            return t._id === updatedTask._id ? updatedTask : t;
        });
    }

    onTaskDeleted(event, taskId) {
        this.rowObject.tasks = this.rowObject.tasks.filter((t) => {
            return t._id !== taskId;
        });
    }
}
